/* ----------------------------------------------------------------------------
 *  $RCSfile: ah.c,v $
 * 
 * $Revision: 1.7 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: ah.c,v $
 * Revision 1.7  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.6  1999/05/20 21:20:07  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.5  1999/05/18 00:47:22  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.4  1999/05/12 13:04:58  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * Revision 1.3  1999/03/27 02:44:17  larse
 * Added support for ICMPv4 type 40 (security failure). CAIRN IPsec does
 * not seem to generate those, however. The -v option is now controls
 * more things and has been debugged. Minor cleanup of the code. Added
 * ipd script as a convenient shortcut, it may go away later.
 *
 * Revision 1.2  1999/03/26 00:59:44  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.1  1999/03/24 00:29:55  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: ah.c,v 1.7 2000/06/07 16:43:17 larse Exp $";


#include <stdio.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"


/* local header id */
static const char hdr[] = "IP-AH";


/* what an ah looks like */
struct auth_hdr
  {
  u_char next_hdr;
  u_char pay_len;
  u_short reserved;
  u_long spi;
  u_long seqno;
  };


/* print an ah */
void print_ah(const u_long destination)
  {
  const struct auth_hdr *ah = 
    (const struct auth_hdr *)next(sizeof(struct auth_hdr));

  if(ah == 0) return;
  print_protocol(hdr, "next header", ah->next_hdr);
  printf("\n");
  iprintf(hdr, "payload length: %hu [%hu octets]\n", ah->pay_len, 
	  (ah->pay_len + 2) * 4);
  iprintf(hdr, "security parameters index: %lu\n", ntohl(ah->spi));
  iprintf(hdr, "sequence number: %lu\n", ntohl(ah->seqno));
  iprintf(hdr, "authentication data: [%hu octets]", (ah->pay_len - 1) * 4);
  if(verbose)
    {
    printf("\n");
    /* print auth data */
    indent_level++;
    print_raw(hdr, (ah->pay_len - 1) * 4);
    indent_level--;
    }
  else 
    {
    /* skip auth data */
    const u_char *skip = next((ah->pay_len - 1) * 4);
    printf(" [omitted]\n");
    skip = 0;
    }
  print_ipv4_content(ah->next_hdr, destination);
  }
