/* ----------------------------------------------------------------------------
 *  $RCSfile: arp.c,v $
 * 
 * $Revision: 1.3 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: arp.c,v $
 * Revision 1.3  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.2  1999/05/20 21:20:07  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.1  1999/05/18 00:47:22  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.1  1999/05/12 13:05:01  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: arp.c,v 1.3 2000/06/07 16:43:17 larse Exp $";


#include <stdio.h>
#include <string.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"

struct  arphdr {
        u_short ar_hrd;         /* format of hardware address */
#define ARPHRD_ETHER    1       /* ethernet hardware format */
#define ARPHRD_FRELAY   15      /* frame relay hardware format */
        u_short ar_pro;         /* format of protocol address */
        u_char  ar_hln;         /* length of hardware address */
        u_char  ar_pln;         /* length of protocol address */
        u_short ar_op;          /* one of: */
#define ARPOP_REQUEST   1       /* request to resolve address */
#define ARPOP_REPLY     2       /* response to previous request */
#define ARPOP_REVREQUEST 3      /* request protocol address given hardware */
#define ARPOP_REVREPLY  4       /* response giving protocol address */
#define ARPOP_INVREQUEST 8      /* request to identify peer */
#define ARPOP_INVREPLY  9       /* response identifying peer */
};

#define ETHER_ADDR_LEN          6
struct  ether_arp {
        struct  arphdr ea_hdr;  /* fixed-size header */
        u_char  arp_sha[ETHER_ADDR_LEN];        /* sender hardware address */
        u_char  arp_spa[4];     /* sender protocol address */
        u_char  arp_tha[ETHER_ADDR_LEN];        /* target hardware address */
        u_char  arp_tpa[4];     /* target protocol address */
};
#define arp_hrd ea_hdr.ar_hrd
#define arp_pro ea_hdr.ar_pro
#define arp_hln ea_hdr.ar_hln
#define arp_pln ea_hdr.ar_pln
#define arp_op  ea_hdr.ar_op

#define ETHERTYPE_IP            0x0800  /* IP protocol */
#define ETHERTYPE_IPV6 0x86dd

/* local header id */
static const char hdr[] = "ARP";


void print_arp(void)
  {
  const struct ether_arp *arp = 
    (struct ether_arp *)next(sizeof(struct ether_arp));
  
  if(arp == 0) return;
  iprintf(hdr, "hardware address format: %hu ", ntohs(arp->arp_hrd));
  switch(ntohs(arp->arp_hrd))
    {
    case ARPHRD_ETHER: printf("[ether]"); break;
    case ARPHRD_FRELAY: printf("[frame relay]"); break;
    default: unsupported++; printf("[*unsupported*]");
    }
  printf("\n");
  iprintf(hdr, "protocol address format: %hu ", ntohs(arp->arp_pro));
  switch(ntohs(arp->arp_pro))
    {
    case ETHERTYPE_IP: printf("[ip]"); break;
    case ETHERTYPE_IPV6: printf("[ipv6]"); break;
    default: unsupported++; printf("[*unsupported*]");
    }
  printf("\n");
  iprintf(hdr, "hardware address length: %hu [octets]\n", arp->arp_hln);
  iprintf(hdr, "protocol address length: %hu [octets]\n", arp->arp_pln);
  iprintf(hdr, "operation: %hu ", ntohs(arp->arp_op));
  switch(ntohs(arp->arp_op))
    {
    case ARPOP_REQUEST: printf("[request]"); break;
    case ARPOP_REPLY: printf("[reply]"); break;
    case ARPOP_REVREQUEST: printf("[reverse request]"); break;
    case ARPOP_REVREPLY: printf("[reverse reply]"); break;
    case ARPOP_INVREQUEST: printf("[inverse request]"); break;
    case ARPOP_INVREPLY: printf("[inverse reply]"); break;
    default: unsupported++; printf("[*unsupported*]");
  }
  printf("\n");
  print_ether_address(hdr, "sender hardware address", arp->arp_sha);
  printf("\n");
  print_address(hdr, "sender protocol address", (u_long *)arp->arp_spa);
  printf("\n");
  print_ether_address(hdr, "target hardware address", arp->arp_tha);
  printf("\n");
  print_address(hdr, "target protocol address", (u_long *)arp->arp_tpa);
  printf("\n");
  }
