/* ----------------------------------------------------------------------------
 *  $RCSfile: ether.c,v $
 * 
 * $Revision: 1.10 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: ether.c,v $
 * Revision 1.10  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.9  1999/08/13 23:50:09  larse
 * Nicer header separation; fixed quirk with printing newlines.
 *
 * Revision 1.8  1999/08/10 22:17:03  larse
 * Now distinguishing between type/length fields, added option to dump raw
 * packets when type is unknown.
 *
 * Revision 1.7  1999/05/20 21:20:07  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.6  1999/05/18 00:47:23  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.5  1999/05/12 13:04:58  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * Revision 1.4  1999/03/26 00:59:45  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.3  1999/03/25 20:23:02  larse
 * Now printing a note when unsupported packet types are
 * encountered. Libpcap location now configurable, this allows
 * compilation with CAIRN's broken libpcap (fixed one in
 * /home/d7unsupp/FreeBSD). Minor other changes.
 *
 * Revision 1.2  1999/03/24 00:29:56  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * Revision 1.1  1999/03/23 03:07:05  larse
 * Added support for TCP and UDP. Cleaned up IP option handling and ICMP.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: ether.c,v 1.10 2000/06/07 16:43:17 larse Exp $";

#include <stdio.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"


#define ETHERTYPE_IPV6 0x86dd
#define ETHER_ADDR_LEN          6

#define ETHERTYPE_IP            0x0800  /* IP protocol */
#define ETHERTYPE_ARP           0x0806  /* Addr. resolution protocol */
#define ETHERTYPE_REVARP        0x8035  /* reverse Addr. resolution protocol */

struct ether_header 
  {
  u_char  ether_dhost[ETHER_ADDR_LEN];
  u_char  ether_shost[ETHER_ADDR_LEN];
  u_short ether_type;
  };


/* local header id */
static const char hdr[] = "ETHER";


/* print an Ethernet header */
void print_ether(void)
  {
  const struct ether_header *eh = 
    (const struct ether_header *)next(sizeof(struct ether_header));

  /* get header (XXX: depends on C headers being present) */
  if(eh == 0) return;
  /* make string from dst ether addr */
  if(print_ll_hdr) 
    {
    print_ether_address(hdr, "destination", eh->ether_dhost);
    printf("\n");
    print_ether_address(hdr, "source", eh->ether_shost);
    printf("\n");
    }
  if(ntohs(eh->ether_type) <= 1500)
    {
    if(print_ll_hdr) 
      iprintf(hdr, "length: %hu [octets]\n", ntohs(eh->ether_type));
    if(verbose) 
      {
      print_seperator();
      print_raw("RAW", ntohs(eh->ether_type));
      }
    if(print_ll_hdr || verbose) printf("\n");
    }
  else
    {
    if(print_ll_hdr) iprintf(hdr, "type: %hu", ntohs(eh->ether_type));
    switch(ntohs(eh->ether_type))
      {
      case ETHERTYPE_IP: 
	if(print_ll_hdr) 
	  {
	  printf(" [IPv4]\n"); 
	  print_seperator();
	  }
	print_ipv4(1);
	printf("\n");
	break;
      case ETHERTYPE_IPV6:
	if(print_ll_hdr) 
	  {
	  printf(" [IPv6]\n"); 
	  print_seperator();
	  }
	break;
      case ETHERTYPE_REVARP:
      case ETHERTYPE_ARP:
	if(print_ll_hdr)
	  {
	  printf(" [ARP]\n"); 
	  print_seperator();
	  }
	print_arp();
	printf("\n");
	break;
      default: 
	unsupported++;
	if(print_ll_hdr) printf(" [*unsupported*]\n");
      }
    }
  }
