/* ----------------------------------------------------------------------------
 *  $RCSfile: icmpv4.c,v $
 * 
 * $Revision: 1.10 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: icmpv4.c,v $
 * Revision 1.10  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.9  1999/05/20 21:20:08  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.8  1999/05/18 00:47:24  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.7  1999/05/12 13:04:59  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * Revision 1.6  1999/03/27 02:44:18  larse
 * Added support for ICMPv4 type 40 (security failure). CAIRN IPsec does
 * not seem to generate those, however. The -v option is now controls
 * more things and has been debugged. Minor cleanup of the code. Added
 * ipd script as a convenient shortcut, it may go away later.
 *
 * Revision 1.5  1999/03/26 00:59:45  larse
 * Now supporting options to inhibit host name lookups, hide link layer
 * headers and show payload data. Documented the code better. Fixed a
 * small bug in IP option handling. First cut at a README.
 *
 * Revision 1.4  1999/03/25 20:23:02  larse
 * Now printing a note when unsupported packet types are
 * encountered. Libpcap location now configurable, this allows
 * compilation with CAIRN's broken libpcap (fixed one in
 * /home/d7unsupp/FreeBSD). Minor other changes.
 *
 * Revision 1.3  1999/03/24 00:29:56  larse
 * Added support for IPIP tunnels, IPsec (AH and ESP) and raw data
 * output. Fixed bugs in the ICMPv4 module. Fixed bug printing certain
 * options in the IP module. Minor changes to other modules, mostly to
 * write raw data where it was omitted previously.
 *
 * Revision 1.2  1999/03/23 03:07:05  larse
 * Added support for TCP and UDP. Cleaned up IP option handling and ICMP.
 *
 * Revision 1.1  1999/03/22 20:20:20  larse
 * First stable and partially working version. IPv4 and ICMPv4 are the
 * only supported protocols, but they work fine, even with
 * extensions. The reorganized code should allow for much easier
 * extensions for other protocols.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: icmpv4.c,v 1.10 2000/06/07 16:43:17 larse Exp $";


#include <stdio.h>
#include <pcap.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"

struct icmp_ra_addr {
        u_long ira_addr;
        u_long ira_preference;
};

struct icmp {
        u_char  icmp_type;              /* type of message, see below */
        u_char  icmp_code;              /* type sub code */
        u_short icmp_cksum;             /* ones complement cksum of struct */
        union {
                u_char ih_pptr;                 /* ICMP_PARAMPROB */
                u_long ih_gwaddr;       /* ICMP_REDIRECT */
                struct ih_idseq {
                        u_short icd_id;
                        u_short icd_seq;
                } ih_idseq;
                int ih_void;

                /* ICMP_UNREACH_NEEDFRAG -- Path MTU Discovery (RFC1191) */
                struct ih_pmtu {
                        u_short ipm_void;
                        u_short ipm_nextmtu;
                } ih_pmtu;

                struct ih_rtradv {
                        u_char irt_num_addrs;
                        u_char irt_wpa;
                        u_short irt_lifetime;
                } ih_rtradv;
        } icmp_hun;
#define icmp_pptr       icmp_hun.ih_pptr
#define icmp_gwaddr     icmp_hun.ih_gwaddr
#define icmp_id         icmp_hun.ih_idseq.icd_id
#define icmp_seq        icmp_hun.ih_idseq.icd_seq
#define icmp_void       icmp_hun.ih_void
#define icmp_pmvoid     icmp_hun.ih_pmtu.ipm_void
#define icmp_nextmtu    icmp_hun.ih_pmtu.ipm_nextmtu
#define icmp_num_addrs  icmp_hun.ih_rtradv.irt_num_addrs
#define icmp_wpa        icmp_hun.ih_rtradv.irt_wpa
#define icmp_lifetime   icmp_hun.ih_rtradv.irt_lifetime
        union {
                struct id_ts {
                        u_long its_otime;
                        u_long its_rtime;
                        u_long its_ttime;
                } id_ts;
                struct icmp_ra_addr id_radv;
                u_long id_mask;
                char    id_data[1];
        } icmp_dun;
#define icmp_otime      icmp_dun.id_ts.its_otime
#define icmp_rtime      icmp_dun.id_ts.its_rtime
#define icmp_ttime      icmp_dun.id_ts.its_ttime
#define icmp_radv       icmp_dun.id_radv
#define icmp_mask       icmp_dun.id_mask
#define icmp_data       icmp_dun.id_data
};


#define ICMP_SEC_FAIL		40 /* as per RFC2521 */
#define ICMP_SEC_FAIL_SPI	 0
#define ICMP_SEC_FAIL_AUTH	 1
#define ICMP_SEC_FAIL_DECOMPR	 2
#define ICMP_SEC_FAIL_DECRPYT	 3
#define ICMP_SEC_FAIL_NEED_AUTHC 4
#define ICMP_SEC_FAIL_NEED_AUTHZ 5

#define ICMP_ECHOREPLY          0               /* echo reply */
#define ICMP_UNREACH            3               /* dest unreachable, codes: */
#define         ICMP_UNREACH_NET        0               /* bad net */
#define         ICMP_UNREACH_HOST       1               /* bad host */
#define         ICMP_UNREACH_PROTOCOL   2               /* bad protocol */
#define         ICMP_UNREACH_PORT       3               /* bad port */
#define         ICMP_UNREACH_NEEDFRAG   4               /* IP_DF caused drop */
#define         ICMP_UNREACH_SRCFAIL    5               /* src route failed */
#define         ICMP_UNREACH_NET_UNKNOWN 6              /* unknown net */
#define         ICMP_UNREACH_HOST_UNKNOWN 7             /* unknown host */
#define         ICMP_UNREACH_ISOLATED   8               /* src host isolated */
#define         ICMP_UNREACH_NET_PROHIB 9               /* prohibited access */
#define         ICMP_UNREACH_HOST_PROHIB 10             /* ditto */
#define         ICMP_UNREACH_TOSNET     11              /* bad tos for net */
#define         ICMP_UNREACH_TOSHOST    12              /* bad tos for host */
#define         ICMP_UNREACH_FILTER_PROHIB 13           /* admin prohib */
#define         ICMP_UNREACH_HOST_PRECEDENCE 14         /* host prec vio. */
#define         ICMP_UNREACH_PRECEDENCE_CUTOFF 15       /* prec cutoff */
#define ICMP_SOURCEQUENCH       4               /* packet lost, slow down */
#define ICMP_REDIRECT           5               /* shorter route, codes: */
#define         ICMP_REDIRECT_NET       0               /* for network */
#define         ICMP_REDIRECT_HOST      1               /* for host */
#define         ICMP_REDIRECT_TOSNET    2               /* for tos and net */
#define         ICMP_REDIRECT_TOSHOST   3               /* for tos and host */
#define ICMP_ECHO               8               /* echo service */
#define ICMP_ROUTERADVERT       9               /* router advertisement */
#define ICMP_ROUTERSOLICIT      10              /* router solicitation */
#define ICMP_TIMXCEED           11              /* time exceeded, code: */
#define         ICMP_TIMXCEED_INTRANS   0               /* ttl==0 in transit */
#define         ICMP_TIMXCEED_REASS     1               /* ttl==0 in reass */
#define ICMP_PARAMPROB          12              /* ip header bad */
#define         ICMP_PARAMPROB_OPTABSENT 1              /* req. opt. absent */
#define ICMP_TSTAMP             13              /* timestamp request */
#define ICMP_TSTAMPREPLY        14              /* timestamp reply */
#define ICMP_IREQ               15              /* information request */
#define ICMP_IREQREPLY          16              /* information reply */
#define ICMP_MASKREQ            17              /* address mask request */
#define ICMP_MASKREPLY          18              /* address mask reply */


/* print header id */
static const char hdr[] = "ICMPv4";


/* common to all headers, avoid code duplication */
static void print_icmpv4_id_seq(const struct icmp *ih)
  {
  iprintf(hdr, "identifier: %hu\n", ntohs(ih->icmp_id));
  iprintf(hdr, "sequence number: %hu\n", ntohs(ih->icmp_seq));
  }


/* print the original packet header for an echo msg */
static void print_icmpv4_orig_pkt(void)
  {
  iprintf(hdr, "original packet header:\n");
  unget(sizeof(struct icmp) - 2 * sizeof(u_long));
  indent_level++;
  print_ipv4(0);
  indent_level--;
  
  if(left())
    {
    iprintf(hdr, "start of original packet data: [%hu octet%s]", 
	    left(), left() == 1 ? "" : "s");
    if(verbose)
      {
      /* print orig packet */
      printf("\n");
      indent_level++;
      print_raw(hdr, left());
      indent_level--;
      }
    else printf(" [omitted]\n");
    }
  }


/* print an icmp header */
void print_icmpv4(void)
  {
  char *code = 0;
  const struct icmp *ih = (const struct icmp *)next(sizeof(struct icmp));

  if(ih == 0) return;
  iprintf(hdr, "type: %hu ", ih->icmp_type);
  switch(ih->icmp_type)
    {
    case ICMP_SEC_FAIL:
      printf("[security failure]\n");
      switch(ih->icmp_code)
	{
	case ICMP_SEC_FAIL_SPI: code = "bad spi"; break;
	case ICMP_SEC_FAIL_AUTH: code = "Authentication Failed"; break;
	case ICMP_SEC_FAIL_DECOMPR: code = "Decompression Failed"; break;
	case ICMP_SEC_FAIL_DECRPYT: code = "Decryption Failed"; break;
	case ICMP_SEC_FAIL_NEED_AUTHC: code = "Need Authentication"; break;
	case ICMP_SEC_FAIL_NEED_AUTHZ: code = "Need Authorization"; break;
	default:
	  unsupported++;
	  code = "[*unsupported*]";
	}
      break;

    case ICMP_ECHOREPLY: printf("[echo reply]\n"); break;
    case ICMP_UNREACH: 
      printf("[destination unreachable]\n"); 
      switch(ih->icmp_code)
	{
	case ICMP_UNREACH_NET: code = "[net unreachable]"; break;
	case ICMP_UNREACH_HOST: code = "[host unreachable]"; break;
	case ICMP_UNREACH_PROTOCOL: code = "[protocol unreachable]"; break;
	case ICMP_UNREACH_PORT: code = "[port unreachable]"; break;
	case ICMP_UNREACH_NEEDFRAG: 
	  code = "[fragmentation needed and DF set]"; 
	  break;
	case ICMP_UNREACH_SRCFAIL: code = "[source route failed]"; break;
	case ICMP_UNREACH_NET_UNKNOWN: code = "[net unknown]"; break;
	case ICMP_UNREACH_HOST_UNKNOWN: code = "[host unknown]"; break;
	case ICMP_UNREACH_ISOLATED: code = "[source host isolated]"; break;
	case ICMP_UNREACH_NET_PROHIB: code = "[host unreachable]"; break;
	case ICMP_UNREACH_HOST_PROHIB: code = "[prohibited access]"; break;
	case ICMP_UNREACH_TOSNET: 
	  code = "[bad type of service for net]"; 
	  break;
	case ICMP_UNREACH_TOSHOST: 
	  code = "[bad type of service for host]"; 
	  break;
	case ICMP_UNREACH_FILTER_PROHIB: 
	  code = "[filter prohibited access]"; 
	  break;
	case ICMP_UNREACH_HOST_PRECEDENCE: 
	  code = "[host precedence violation]"; 
	  break;
	case ICMP_UNREACH_PRECEDENCE_CUTOFF: 
	  code = "[precedence cutoff]"; 
	  break;
	default:
	  unsupported++;
	  code = "[*unsupported*]";
	}
      break;

    case ICMP_SOURCEQUENCH: printf("[source quench]\n"); break;
    case ICMP_REDIRECT: 
      printf("[redirect]\n"); 
      switch(ih->icmp_code)
	{
	case ICMP_REDIRECT_NET: 
	  code = "[redirect datagrams for network]"; 
	  break;
	case ICMP_REDIRECT_HOST: 
	  code = "[redirect datagrams for host]"; 
	  break;
	case ICMP_REDIRECT_TOSNET: 
	  code = "[redirect datagrams for type of service and network]"; 
	  break;
	case ICMP_REDIRECT_TOSHOST: 
	  code = "[redirect datagrams for type of service and host]"; 
	  break;
	default: 
	  unsupported++;
	  code = "[*unsupported*]";
	}
      break;
      
    case ICMP_ECHO: printf("[echo]\n"); break;
    case ICMP_ROUTERADVERT: printf("[router advertisement]\n"); break;
    case ICMP_ROUTERSOLICIT: printf("[router solicitation]\n"); break;
    case ICMP_TIMXCEED: 
      printf("[time exceeded]\n"); 
      switch(ih->icmp_code)
	{
	case ICMP_TIMXCEED_INTRANS:
	  code = "[time to live exceeded in transit]";
	  break;
	case ICMP_TIMXCEED_REASS:
	  code = "[fragment reassembly time exceeded]";
	  break;
	default:
	  unsupported++;
	  code = "[*unsupported*]";
	}
      break;

    case ICMP_PARAMPROB: 
      printf("[parameter problem]\n");
      switch(*code)
	{
	case ICMP_PARAMPROB_OPTABSENT:
	  code = "[required option absent]";
	  break;
	default:
	  unsupported++;
	  code = "[*unsupported*]";
	}
      break;

    case ICMP_TSTAMP: printf("[timestamp request]\n"); break;
    case ICMP_TSTAMPREPLY: printf("[timestamp reply]\n"); break;
    case ICMP_IREQ: printf("[information request]\n"); break;
    case ICMP_IREQREPLY: printf("[information reply]\n"); break;
    case ICMP_MASKREQ: printf("[address mask request]\n"); break;
    case ICMP_MASKREPLY: printf("[address mask reply]\n"); break;
    default: 
      unsupported++;
      printf("[*unsupported*]\n");
    }

  iprintf(hdr, "code: %hu %s\n", ih->icmp_code, code ? code : "");
  iprintf(hdr, "checksum: %hu\n", ntohs(ih->icmp_cksum));

  /* print remainder of header based on type */
  switch(ih->icmp_type)
    {
    case ICMP_SEC_FAIL:
      /* icmp_seq is not the right field name, but has right length and pos */
      iprintf(hdr, "pointer: %hu\n", ntohs(ih->icmp_seq));
      print_icmpv4_orig_pkt();
      break;

    case ICMP_ROUTERADVERT:
      iprintf(hdr, "number of addresses: %hu\n", ih->icmp_num_addrs);
      iprintf(hdr, "address entry size: %hu [%hu octets]\n", ih->icmp_wpa,
	      ih->icmp_wpa * 4);
      iprintf(hdr, "lifetime: %hu [second%s]\n", ntohs(ih->icmp_lifetime),
	      ntohs(ih->icmp_lifetime) == 1 ? "" : "s");
      iprintf(hdr, "router addresses:\n");
      indent_level++;
      if(ih->icmp_wpa == 2)
	{
	u_short n;

	for(n = 0; n < ih->icmp_num_addrs; n++)
	  {
	    const struct icmp_ra_addr *raddr = (const struct icmp_ra_addr *)
	      next(sizeof(struct icmp_ra_addr));
	  if(raddr == 0) return;
	  print_address(hdr, 0, (u_long *)&raddr->ira_addr);
	  printf(" %ld\n", (long)ntohl(raddr->ira_preference));
	  }
	}
      else iprintf(hdr, "[omitted; can only handle entries of size 2]\n");
      indent_level--;
      break;

    case ICMP_MASKREQ:
    case ICMP_MASKREPLY:
      print_icmpv4_id_seq(ih);
      print_address(hdr, "address mask", &ih->icmp_mask);
      printf("\n");
      break;


    case ICMP_TSTAMP:
    case ICMP_TSTAMPREPLY:
      print_icmpv4_id_seq(ih);
      iprintf(hdr, "originate timestamp: %ld\n", ntohl(ih->icmp_otime));
      iprintf(hdr, "receive timestamp: %ld\n", ntohl(ih->icmp_rtime));
      iprintf(hdr, "transmit timestamp: %ld\n", ntohl(ih->icmp_ttime));
      break;

    case ICMP_IREQ:
    case ICMP_IREQREPLY:
      print_icmpv4_id_seq(ih);
      break;

    case ICMP_ECHO: 
    case ICMP_ECHOREPLY:
      print_icmpv4_id_seq(ih);
      unget(sizeof(struct icmp) - 2 * sizeof(u_long));
      if(left())
	{
	iprintf(hdr, "data: [%hu octet%s]", left(), left() == 1 ? "" : "s");
	if(verbose)
	  {
	  /* print icmp data */
	  printf("\n");
	  indent_level++;
	  print_raw(hdr, left());
	  indent_level--;
	  }
	else printf(" [omitted]\n");
	}
      break;
    
    case ICMP_PARAMPROB:
      iprintf(hdr, "pointer: %hu\n", ih->icmp_pptr);
      print_icmpv4_orig_pkt();
      break;

    case ICMP_REDIRECT: 
      print_address(hdr, "gateway address", (u_long *)&ih->icmp_gwaddr);
      printf("\n");
      print_icmpv4_orig_pkt();
      break;

    case ICMP_UNREACH:
      if(ih->icmp_code == ICMP_UNREACH_NEEDFRAG)
	iprintf(hdr, "next hop mtu: %hu\n", ntohs(ih->icmp_nextmtu));

    case ICMP_TIMXCEED: 
    case ICMP_SOURCEQUENCH:
      print_icmpv4_orig_pkt();
      break;
    }
  }
