/* ----------------------------------------------------------------------------
 *  $RCSfile: sap.c,v $
 * 
 * $Revision: 1.3 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:18 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: sap.c,v $
 * Revision 1.3  2000/06/07 16:43:18  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.2  1999/05/18 00:47:25  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.1  1999/05/12 13:05:00  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: sap.c,v 1.3 2000/06/07 16:43:18 larse Exp $";


#include <stdio.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"


/* local header id */
static const char hdr[] = "SAP";


struct sap_hdr
  {
  u_char v:3,
         a:1,
         mt:2,
         flags:2;
  u_char auth_len;
  u_short msg_id_hash;
  u_long orig_src;
  };

#define SAP_ENCRYPT 2
#define SAP_COMPRESSED 1
#define SAP_FLAGS (SAP_ENCRYPT|SAP_COMPRESSED)


/* print sap */
void print_sap(void)
  {
  const struct sap_hdr *sap = (struct sap_hdr *)next(sizeof(struct sap_hdr));
  int fl = 0;
  
  if(sap == 0) return;
  iprintf(hdr, "version: %hu\n", sap->v);
  iprintf(hdr, "address type: %hu ", sap->a);
  if(sap->a == 0)
    {
    printf("[ipv4]\n");
    iprintf(hdr, "message type: %hu ", sap->mt);
    switch(sap->mt)
      {
      case 0: printf("[announcement]\n"); break;
      case 1: printf("[deletion]\n"); break;
      default:
	unsupported++;
	printf("[*unsupported*]\n");
      }
    iprintf(hdr, "flags: %hu ", sap->flags);
    if(sap->flags & SAP_FLAGS)
      {
      printf(" [");
      if(sap->flags & SAP_ENCRYPT) printf("%sencrypted", fl++ ? "|" : "");
      if(sap->flags & SAP_COMPRESSED) printf("%scompressed", fl++ ? "|" : "");
      printf("]\n");
      }
    else printf("\n");
    iprintf(hdr, "authentication length: %hu\n", sap->auth_len);
    iprintf(hdr, "message identifier hash: %hu\n", ntohs(sap->msg_id_hash));
    print_address(hdr, "originating source", &sap->orig_src);
    printf("\n");
    if(sap->auth_len)
      {
      iprintf(hdr, "authentication header: [%hu octet%s]", sap->auth_len, 
	      sap->auth_len == 1 ? "" : "s");
      if(verbose)
	{
	printf("\n");
	indent_level++;
	print_raw(hdr, sap->auth_len);
	indent_level--;
	}
      else printf(" [omitted]\n");
      }
    if(sap->flags & 2)
      {
      const u_long *l = (u_long *)next(sizeof(u_long));
      if(l == 0) return;
      iprintf(hdr, "key id: %lu ", ntohl(*l));
      l = (u_long *)next(sizeof(u_long));
      if(l == 0) return;      
      iprintf(hdr, "timeout: %lu ", ntohl(*l));
      l = (u_long *)next(sizeof(u_long));
      if(l == 0) return;      
      iprintf(hdr, "payload padded: %s", ntohl(*l) >> 31 ? "yes" : "no");
      iprintf(hdr, "random field: %lu ", ntohl(*l) && 0x3fffffff);
      }
    if(sap->flags & SAP_ENCRYPT) iprintf(hdr, "payload: [encrypted]");
    else if(sap->flags & SAP_ENCRYPT) iprintf(hdr, "payload: [compressed]");
    else if(sap->v > 1)
      {
      unsupported++;
      iprintf(hdr, "payload: [*unsupported*]\n");
      }
    else print_sdp();
    }
  else
    {
    unsupported++;
    printf("[*unsupported*]\n");
    }
  }
